


import org.apache.tools.ant.Project

import com.sas.metadata.remote.*
import com.sas.batch.localsvc.ScriptingLocalServicesContext;
import com.sas.services.user.UserContextInterface;
import com.sas.services.information.metadata.servers.LASRServerUtil;


class MetadataUtilityClass {

	def MdFactoryImpl factory
	def MdOMRConnection omrConnection
	def MdObjectStore mdObjectStore
	def project
	def fqId
	def reposId

	def debug = true

	/**
	 * Constructor
	 */
	public MetadataUtilityClass(project) {
		super()
		this.project = project
	}

	MdOMRConnection makeConnection(host, port, user, password, reposName) {
		factory = new MdFactoryImpl(false)
		factory.setDebug(true)

		if (debug) {
			// Enable logging
			factory.setLoggingEnabled(true)
			// Set the output streams for logging.  The logging output can be
			// directed to any OutputStream, including a file.
			factory.getUtil().setOutputStream(System.out)
			factory.getUtil().setLogStream(System.out)
		}

		factory.makeOMRConnection(host, port, user, password);
		omrConnection = factory.getConnection();
		if (omrConnection == null) {
			e1 = new MdException("Error connecting to metadata server.")
			throw e1
		} else {

			Utility.antEcho(project, "Connected to metadata server.", debug)
		}
		mdObjectStore = factory.createObjectStore()
		extractRposFqid(reposName)
		return omrConnection
	}

	void extractRposFqid(reposName) {
		def MdRepositoryUtil repositories = factory.getRepositoryUtil()
		fqId = repositories.getRepositoryIDFromName(reposName)
		repositories.dispose()
		if (fqId == null) {
			e1 = new MdException("Error getting ID for repository, ${reposName}.")
			throw e1
		} else {
			reposId=fqId.substring(9)
		}
	}

	void closeConnection() {
		if (mdObjectStore != null) {
			try {
				mdObjectStore.dispose()
			}
			catch (Exception e99) {

				Utility.antEcho(project, "closeConnection: " + e99.getMessage(), debug)
			}
		}
		if (omrConnection != null) {
			try {
				omrConnection.closeOMRConnection()
			}
			catch (Exception e) {
				Utility.antEcho(project, "closeConnection: " + e.getMessage(), debug)
			}
		}
	}

	List findMetadataObjects(mdType, xmlSelect) {
		def MdOMIUtil util = factory.getOMIUtil()
		List objectList = util.getMetadataObjectsSubset(mdObjectStore, fqId, mdType,
				  util.OMI_XMLSELECT
				| util.OMI_GET_METADATA
				| util.OMI_ALL
				| util.OMI_ALL_SIMPLE,
				xmlSelect
				)
		return objectList
	}

	List findMetadataRootObjects(xmlSelect) {
		def mdType = MetadataObjects.ROOT
		def MdOMIUtil util = factory.getOMIUtil()
		List objectList = util.getMetadataObjectsSubset(mdObjectStore, fqId, mdType,
				  util.OMI_XMLSELECT
				| util.OMI_GET_METADATA
				| util.OMI_ALL
				| util.OMI_ALL_SIMPLE
				| util.OMI_INCLUDE_SUBTYPES,
				xmlSelect
				)
		return objectList
	}

	void deleteObject(object) {
		factory.deleteMetadataObject(object)
	}

	Root findObject(fqid) {
		def xmlSelect= "<XMLSELECT Search=\"Root[@Id='${fqid}']\"/>"
		def mdObject
		def found = findMetadataRootObjects(xmlSelect);
		if (found != null && found.size() > 0) {
			mdObject = (Root)found.get(0)
		}
		return mdObject
	}

	SASLibrary findLibrary(searchPath) {
		def mdType = MetadataObjects.SASLIBRARY
		def xmlSelect= "<XMLSELECT Search=\"${mdType}${searchPath}\"/>"
		def mdObject
		def found = findMetadataObjects(mdType, xmlSelect);
		if (found != null && found.size() > 0) {
			mdObject = (SASLibrary)found.get(0)
		}
		return mdObject
	}

	IdentityGroup getGroupByFqid(fqid) {
		def mdType = MetadataObjects.IDENTITYGROUP
		def xmlSelect= "<XMLSELECT Search=\"${mdType}[@Id='${fqid}']\"/>"
		def mdObject
		def found = findMetadataObjects(mdType, xmlSelect);
		if (found != null && found.size() > 0) {
			mdObject = (IdentityGroup)found.get(0)
		}
		return mdObject
	}

	IdentityGroup getGroupByName(name) {
		def mdType = MetadataObjects.IDENTITYGROUP
		def xmlSelect= "<XMLSELECT Search=\"${mdType}[@Name='${name}']\"/>"
		def mdObject
		def found = findMetadataObjects(mdType, xmlSelect);
		if (found != null && found.size() > 0) {
			mdObject = (IdentityGroup)found.get(0)
		}
		return mdObject
	}

	Machine getMachine(name) {
		String mdType = MetadataObjects.MACHINE
		def xmlSelect = "<XMLSELECT Search=\"@Name='${name}'\"/>"
		def mdObject
		List found = findMetadataObjects(mdType, xmlSelect);
		if (found != null && found.size() > 0) {
			mdObject = (Machine)found.get(0)
		}
		return mdObject
	}

	AuthenticationDomain getAuthDomain(name) {
		def mdType = MetadataObjects.AUTHENTICATIONDOMAIN
		def xmlSelect = "<XMLSELECT Search=\"@Name='${name}'\"/>"
		def mdObject
		List found = findMetadataObjects(mdType, xmlSelect)
		if (found != null && found.size() > 0) {
			mdObject = (AuthenticationDomain)found.get(0)
		}
		return mdObject
	}

	Login getLogin(userId) {
		def mdType = MetadataObjects.LOGIN
		def xmlSelect = "<XMLSELECT Search=\"@UserID='${userId}'\"/>"
		def mdObject
		List found = findMetadataObjects(mdType, xmlSelect)
		if (found != null && found.size() > 0) {
			mdObject = (Login)found.get(0)
		}
		return mdObject
	}

	Login getLoginWithDomain(userId, domain) {
		def mdType = MetadataObjects.LOGIN
		def xmlSelect = "<XMLSELECT Search=\"${mdType}[@UserID='${userId}'][Domain/*[@Name='${domain}']]\"/>"
		def mdObject
		List found = findMetadataObjects(mdType, xmlSelect)
		if (found != null && found.size() > 0) {
			mdObject = (Login)found.get(0)
		}
		return mdObject
	}

	SASClientConnection getSCConnection(name) {
		def mdType = MetadataObjects.SASCLIENTCONNECTION
		def xmlSelect = "<XMLSELECT Search=\"@Name='${name}'\"/>"
		def mdObject
		List found = findMetadataObjects(mdType, xmlSelect)
		if (found != null && found.size() > 0) {
			mdObject = (SASClientConnection)found.get(0)
		}
		return mdObject
	}

    SASClientConnection getConnectionToServer(name, serverName) {
        def mdType = MetadataObjects.SASCLIENTCONNECTION
        def xmlSelect = "<XMLSELECT Search=\"${mdType}[@Name='${name}'][Source/*[@Name='${serverName}']]\"/>"
        def mdObject
        List found = findMetadataObjects(mdType, xmlSelect)
        if (found != null && found.size() > 0) {
            mdObject = (SASClientConnection)found.get(0)
        }
        return mdObject
    }

	ServerContext getServerContext(name) {
		def mdType = MetadataObjects.SERVERCONTEXT
		def xmlSelect = "<XMLSELECT Search=\"@Name='${name}'\"/>"
		def mdObject
		List found = findMetadataObjects(mdType, xmlSelect)
		if (found != null && found.size() > 0) {
			mdObject = (ServerContext)found.get(0)
		}
		return mdObject
	}

	SASLibrary getSasLibrary(name) {
		def mdType = MetadataObjects.SASLIBRARY
		def xmlSelect = "<XMLSELECT Search=\"@Name='${name}'\"/>"
		def mdObject
		List found = findMetadataObjects(mdType, xmlSelect)
		if (found != null && found.size() > 0) {
			mdObject = (SASLibrary)found.get(0)
		}
		return mdObject
	}

	Tree getMetadataFolder(searchString) {
		def mdType = MetadataObjects.TREE
		def xmlSelect = "<XMLSELECT Search=\"${mdType}${searchString}\"/>"
		def mdObject
		List found = findMetadataObjects(mdType, xmlSelect)
		if (found != null && found.size() > 0) {
			mdObject = (Tree)found.get(0)
		}
		return mdObject
	}

	Property getProperty(name) {
		def mdType = MetadataObjects.PROPERTY
		def xmlSelect = "<XMLSELECT Search=\"@Name='${name}'\"/>"
		def mdObject
		List found = findMetadataObjects(mdType, xmlSelect)
		if (found != null && found.size() > 0) {
			mdObject = (Property)found.get(0)
		}
		return mdObject
	}

	PropertySet getPropertySet(name, libname) {
		def mdType = MetadataObjects.PROPERTYSET
		def xmlSelect = "<XMLSELECT Search=\"${mdType}[@Name='${name}'][OwningObject/*[@Name='${libname}']\"/>"
		def mdObject
		List found = findMetadataObjects(mdType, xmlSelect)
		if (found != null && found.size() > 0) {
			mdObject = (PropertySet)found.get(0)
		}
		return mdObject
	}

	ServerComponent getServerComponent(name){
		def mdType = MetadataObjects.SERVERCOMPONENT
		def xmlSelect = "<XMLSELECT Search=\"@Name='${name}'\"/>"
		def sComp
		List found = findMetadataObjects(mdType, xmlSelect)
		if (found != null && found.size() > 0) {
			sComp = (ServerComponent)found.get(0)
		}
		return sComp
	}

	Directory getDirectory(name){
		def mdType = MetadataObjects.DIRECTORY
		def xmlSelect = "<XMLSELECT Search=\"@Name='${name}'\"/>"
		def mdDir
		List found = findMetadataObjects(mdType, xmlSelect)
		if (found != null && found.size() > 0) {
			mdDir = (Directory)found.get(0)
		}
		return mdDir
	}

	ServerComponent makeServerComponent(name) {
		def type = MetadataObjects.SERVERCOMPONENT
		def newMdObject = (ServerComponent)factory.createComplexMetadataObject(mdObjectStore, null, name, type, reposId)
		return newMdObject
	}

	SASClientConnection makeSasClientConnection(name) {
		def type = MetadataObjects.SASCLIENTCONNECTION
		def newMdObject = (SASClientConnection)factory.createComplexMetadataObject(mdObjectStore, null, name, type, reposId)
		return newMdObject
	}

	TCPIPConnection makeTCPIPConnection(name) {
		def type = MetadataObjects.TCPIPCONNECTION
		def newMdObject = (TCPIPConnection)factory.createComplexMetadataObject(mdObjectStore, null, name, type, reposId)
		return newMdObject
	}

	AssociationList makeServiceType(name, association) {
		def type = MetadataObjects.SERVICETYPE
		def newMdObject = factory.createComplexMetadataObject(mdObjectStore, null, name, type, reposId)
		def aList = new AssociationList(association, 1)
		aList.add(newMdObject)
		return aList
	}

	SASLibrary makeSasLibrary(name) {
		def type = MetadataObjects.SASLIBRARY
		def newMdObject = (SASLibrary)factory.createComplexMetadataObject(mdObjectStore, null, name, type, reposId)
		return newMdObject
	}

	TextStore makeTextStore(name) {
		def type = MetadataObjects.TEXTSTORE
		def newMdObject = (TextStore)factory.createComplexMetadataObject(mdObjectStore, null, name, type, reposId)
		return newMdObject
	}

	DatabaseSchema makeDatabaseSchema(name) {
		def type = MetadataObjects.DATABASESCHEMA
		def newMdObject = (DatabaseSchema)factory.createComplexMetadataObject(mdObjectStore, null, name, type, reposId)
		return newMdObject
	}

	PropertySet makePropertySet(name) {
		def type = MetadataObjects.PROPERTYSET
		def newMdObject = (PropertySet)factory.createComplexMetadataObject(mdObjectStore, null, name, type, reposId)
		return newMdObject
	}

   Property makeNewSetProperty(t, name, defaultValue) {
      // Named arguments: propertyName:"Text"
      if (t.propertyName == null) { t.propertyName = name }
      if (t.sqlType == null) { t.sqlType = 12 }
      if (t.delimiter == null) { t.delimiter = "" }
      if (t.useValueOnly == null) { t.useValueOnly = "0" }
      // Create the property
      def type = MetadataObjects.PROPERTY
      def newProperty = (Property)factory.createComplexMetadataObject(mdObjectStore, null, name, type, reposId)
      println "New property \"${name}\" created."
      // Set values
      newProperty.with {
          setPropertyName t.propertyName
          setSQLType t.sqlType
          setDefaultValue defaultValue
          setUseValueOnly t.useValueOnly
      }
      println "Attributes set."
      if (!t.delimiter.equals("")) {
          newProperty.setDelimiter(t.delimiter)
      }

      return newProperty
   }

	Machine makeNewMachine(name) {
		def type = MetadataObjects.MACHINE
		def newMdObject = (Machine)factory.createComplexMetadataObject(mdObjectStore, null, name, type, reposId)
		return newMdObject
	}

	Directory makeDirectory(name) {
		def type = MetadataObjects.DIRECTORY
		def newMdObject = (Directory)factory.createComplexMetadataObject(mdObjectStore, null, name, type, reposId)
		return newMdObject
	}

	AuthenticationDomain makeAuthenticationDomain(name) {
		def type = MetadataObjects.AUTHENTICATIONDOMAIN
		def newMdObject = (AuthenticationDomain)factory.createComplexMetadataObject(mdObjectStore, null, name, type, reposId)
		return newMdObject
	}

	Login makeLogin(name) {
		def type = MetadataObjects.LOGIN
		def newMdObject = (Login)factory.createComplexMetadataObject(mdObjectStore, null, name, type, reposId)
		return newMdObject
	}

	Property makeNewProperty(t, name, defaultValue) {
		// Named arguments: propertyName:"Text"
		if (t.propertyName == null) { t.propertyName = name }
		if (t.sqlType == null) { t.sqlType = 12 }
		if (t.propertyType == null) {
			if (t.sqlType == 4){
				t.propertyType="Integer"
			} else {
				t.propertyType="String"
			}
		}
		if (t.delimiter == null) { t.delimiter = "" }
		if (t.useValueOnly == null) { t.useValueOnly = "0" }
		// Create the property
		def type = MetadataObjects.PROPERTY
		def newProperty = (Property)factory.createComplexMetadataObject(mdObjectStore, null, name, type, reposId)
		println "New property \"${name}\" created with defaultValue \"${defaultValue}\" with PropertyName \"${t.propertyName}\" with delimiter \"${t.delimiter}\"."
		// Set values
		newProperty.with {
			setPropertyName t.propertyName
			setSQLType t.sqlType
			setDefaultValue defaultValue
			setUseValueOnly t.useValueOnly
		}
		println "Attributes set."
		if (!t.delimiter.equals("")) {
			newProperty.setDelimiter(t.delimiter)
		}
		// Owning Type
		type = MetadataObjects.PROPERTYTYPE
		def newPropertyType = (PropertyType)factory.createComplexMetadataObject(mdObjectStore, null, t.propertyType, type, reposId)
		println "New property type \"${t.propertyType}\" created."
		newPropertyType.setSQLType(t.sqlType)
		println "SQL Type set."
		if (t.propertyType.equals("Yes, No, or Blank Types")) {
			def text = mkYesNoBlankText()
			println "Text Store created."
			newPropertyType.setStoredConfiguration(text)
			println "Text Store added."
		} else if (t.propertyType.equals("Yes or No Types")) {
			def text = mkYesNoText()
			println "Text Store created."
			newPropertyType.setStoredConfiguration(text)
			println "Text Store added."
		} else if (t.propertyType.equals("Connection Types")) {
			def text = mkConnectionTypesText()
			println "Text Store created."
			newPropertyType.setStoredConfiguration(text)
			println "Text Store added."
        }
		newProperty.setOwningType(newPropertyType)
		println "Property Type added."
		return newProperty
	}

   void setThisProperty(name, propName, value, delim, properties, force=false)
   {
       // To be backwards compatible, if being invoked with no named parameters, simply
       // make up a new named parameter, so that we can keep all the logic in a single method.
       setThisProperty(name, propName, value, delim, properties, force, bogusName:"na")
   }

   // force=true means to create the property anyway, when no value is given.
   // force=false means to remove the property, when no value is given.
   void setThisProperty(Map t, name, propName, value, delim, properties, force=false)
   {
      if ( t.overwriteValue == null )
      {
         t.overwriteValue = "true"
      }

      if ( t.propTypeName == null )
      {
         t.propTypeName = "String"
      }

      if ( t.propTypeSQL == null )
      {
         t.propTypeSQL = "12"
      }

      if (((value != null) && (!value.equals(""))) | force)
      {
         def property = properties.find { it.getName().equals(name) }
         if (property != null)
         {
            if ( t.overwriteValue.equalsIgnoreCase("true") || t.overwriteValue.equalsIgnoreCase("yes") )
            {
               (value == null) ? "" : value
               property.setDefaultValue(value)
            }
            else
            {
               def existingValue = property.getDefaultValue()
               Utility.antEcho(project, "MetadataUtilityClass.setThisProperty():  Property name '${name}' was found, but did NOT override the existing value of '${existingValue}' with '${value}'.", debug)
            }
         }
         else
         {
            property = makeNewProperty(name, value, propertyName:propName, delimiter:delim, propertyType:t.propTypeName, sqlType:t.propTypeSQL)
         }
         properties.add(property)
      }
      else
      {
         def property = properties.find { it.getName().equals(name) }
         if (property != null)
         {
            properties.remove(property)
            deleteObject(property)
         }
      }
   }

	TextStore mkYesNoBlankText() {
		def name = "Enumeration of Yes, No, or Blank"
		def textStore = makeTextStore(name)
		def storedText = "<Configuration>                <Enumeration>                    <Value name=\"\">Library.Common.Value.BlankNoValue.xmlKey.txt</Value>                    <Value name=\"YES\">Library.DBMS.Value.Yes.xmlKey.txt</Value>                    <Value name=\"NO\">Library.DBMS.Value.No.xmlKey.txt</Value>                 </Enumeration>             </Configuration>"
		textStore.with {
			setStoredText storedText
			setTextRole "config"
			setTextType "xml"
			setUsageVersion 0
		}
		return textStore
	}

	TextStore mkYesNoText() {
		def name = "Enumeration of Yes or No Types"
		def textStore = makeTextStore(name)
		def storedText = "<Configuration> <Enumeration>              <Value name=\"YES\">Library.DBMS.Value.Yes.xmlKey.txt</Value>             <Value name=\"NO\">Library.DBMS.Value.No.xmlKey.txt</Value>           </Enumeration> </Configuration>"
		textStore.with {
			setStoredText storedText
			setTextRole "config"
			setTextType "xml"
			setUsageVersion 0
		}
		return textStore
	}

	TextStore mkConnectionTypesText() {
		def name = "Enumeration of Connection Types"
		def textStore = makeTextStore(name)
		def storedText = "<Configuration> <Enumeration> <Value name=\"SHAREDREAD\">Library.DBMS.ConnectType.SharedRead.xmlKey.txt</Value> <Value name=\"GLOBALREAD\">Library.DBMS.ConnectType.GlobalRead.xmlKey.txt</Value> <Value name=\"UNIQUE\">Library.DBMS.ConnectType.Unique.xmlKey.txt</Value> <Value name=\"SHARED\">Library.DBMS.ConnectType.Shared.xmlKey.txt</Value> <Value name=\"GLOBAL\">Library.DBMS.ConnectType.Global.xmlKey.txt</Value> </Enumeration> </Configuration> "
		textStore.with {
			setStoredText storedText
			setTextRole "config"
			setTextType "xml"
			setUsageVersion 0
		}
		return textStore
	}

   String getLASRSignerURL(host, port, user, password) {

      ScriptingLocalServicesContext context = new ScriptingLocalServicesContext("LASRLibraryConfiguration");
      context.setConnection(host, port);
      context.setCredential(user, password);
      context.setUp();
      UserContextInterface usercontext = context.getUser();
      String signerURL = LASRServerUtil.getAuthorizationServiceUrl(usercontext);

      return signerURL;
   }

   Extension makeNewExtension(name, value) {
      def type = MetadataObjects.EXTENSION
      def newMdObject = (Extension)factory.createComplexMetadataObject(mdObjectStore, null, name, type, reposId)
      newMdObject.setValue(value)
      return newMdObject
   }

   void updateExtensionInAssocList(extensionName, extensionValue, assocListOfExtensions) {
      Boolean foundExtension = false

      for (Iterator it = assocListOfExtensions.iterator(); it.hasNext();)
      {
         Extension extension = (Extension)it.next();
         if (extension.getName().equals(extensionName))
         {
            foundExtension=true
            extension.setValue(extensionValue)
         }
      }

      if (false == foundExtension)
      {
         Extension newExtension = makeNewExtension(extensionName, extensionValue)
         assocListOfExtensions.add(newExtension)
      }

      return
   }

}
